@@ -32,27 +32,26 @@
 ;; Parse given a token generator.  Uses fluid @code{*info*}.
 ;; A little ugly wrt re-throw but
 (define raw-parser
-  (let ((c99-parser (make-lalr-parser
+  (let ((parser (make-lalr-parser
 		     (list (cons 'len-v len-v) (cons 'pat-v pat-v)
 			   (cons 'rto-v rto-v) (cons 'mtab mtab)
 			   (cons 'act-v act-v)))))
     (lambda* (lexer #:key (debug #f))
       (catch
        'nyacc-error
-       (lambda () (c99-parser lexer #:debug debug))
+       (lambda () (parser lexer #:debug debug))
        (lambda (key fmt . args)
 	 (report-error fmt args)
 	 (pop-input)			; not sure this is the right way
-	 (throw 'c99-error "C99 parse error")))
-      )))
+	 (throw 'c99-error "C99 parse error"))))))
 
 ;; This is used to parse included files at top level.
 (define (run-parse)
   (let ((info (fluid-ref *info*)))
-    (raw-parser (gen-c-lexer) #:debug (cpi-debug info))))
+    (raw-parser (gen-c-lexer #:mode 'decl) #:debug (cpi-debug info))))
 
-;; @deffn parse-c99 [#:cpp-defs def-a-list] [#:inc-dirs dir-list] \
-;;               [#:mode ('code|'file)] [#:debug bool]
+;; @deffn {Procedure} parse-c99 [#:cpp-defs def-a-list] [#:inc-dirs dir-list] \
+;;               [#:mode ('code|'file|'decl)] [#:debug bool]
 ;; This needs to be explained in some detail.
 ;; tdd = typedef dict: (("<time>" time_t) ... ("<unistd.h>" ...))
 ;; Default mode is @code{'code}.
@@ -63,11 +62,15 @@
 ;;            #:inc-help (append '("myinc.h" "foo_t" "bar_t") c99-std-help)
 ;;            #:mode 'file))
 ;; @end example
+;; Note: for @code{file} mode user still needs to make sure CPP conditional
+;; expressions can be fully evaluated, which may mean adding compiler generated
+;; defines (e.g., using @code{gen-cpp-defs}).
+;; @end deffn
 (define* (parse-c99 #:key
 		    (cpp-defs '())	; CPP defines
 		    (inc-dirs '())	; include dirs
 		    (inc-help '())	; include helpers
-		    (mode 'code)	; mode: 'file or 'code
+		    (mode 'code)	; mode: 'file, 'code or 'decl
 		    (xdef? #f)		; pred to determine expand
 		    (debug #f))		; debug
   (catch
